from .llm import query_llm
from .main import main

from .agents.airbnb_review_agent import airbnb_review_agent
from .agents.web_search_agent import web_search_agent
from .agents.web_scraper_agent import web_scraper_agent


"""Read all code files in current folder and turn into prompts."""

from pathlib import Path

from .util import assert_file_exists_and_read

def load_code_folder_to_system_prompt(path):
    file_contents = {}
    file_types = ['.py', '.html', '.css', '.js', '.sh']
    
    for file_path in Path(path).iterdir():
        if file_path.suffix in file_types:
            file_contents[file_path.name] = assert_file_exists_and_read(file_path)

    content = "# Example Project\n"
    content += "This is the example project which contains the following files:\n\n"

    for filename, file_content in file_contents.items():
        content += f"## {filename}\n"
        content += f"```{filename.split('.')[-1]}\n"
        content += file_content
        content += "\n```\n\n"

    content += "# Objective\n"
    content += "Modify and re-generate any of these files (if necessary) based on new <FEATURE_REQUEST></FEATURE_REQUEST> statements.\n\n"

    content += "# Output\n"
    content += "Output new or modified file contents using the following format:\n"
    content += "<filename_extension>File content goes here</filename_extension>\n"
    content += "For example:\n"
    content += "<myfile_py>print('Hello, World!')</myfile_py>\n"
    content += "This will create or update a file named 'myfile.py' with the given content.\n\n"
    content += "## Note\n"
    content += "1. Only output one file in each conversation round, based on the user query.\n"
    content += "2. Once a file is updated, always print the full content (no diff format).\n"
    content += "3. When no file is to update, output \"===end of generation===\".\n"

    return content



import os
import random

from .llm import query_llm
from .produce_files import produce_files
from .util import assert_file_exists_and_read, load_history, save_history

PORT = random.randint(5000, 10000)


DESIGN_SYSTEM_PROMPT = f"""You are Claude, an AI assistant powered by Anthropic's Claude-3.5-Sonnet model, specialized in software development.
You are a software architecture.
Your job to design an app. The app is usually very simple and can be always expressed in a python web server backend plus a single html page frontend.
The given input is the app description.
Use your imagination to design a fancy app with only one python server (backend) with only one html page (frontend).
You only write the design doc with high-level structure and psudocode, no need to write actual code.  

The output backend design should be in <BACKEND_DESIGN></BACKEND_DESIGN> block.
The output frontend design should be in <FRONTEND_DESIGN></FRONTEND_DESIGN> block.
There should be exactly one <BACKEND_DESIGN> and one <FRONTEND_DESIGN>.
"""

SERVER_SYSTEM_PROMPT = f"""You are Claude, an AI assistant powered by Anthropic's Claude-3.5-Sonnet model, specialized in backend development.
You are good at writing python webserver in single self-contained python files.

1. Regarding data or database.
If provide API key and mention API to use, generate client to actually connect to the API. (assume API is accesible and key is correct.)
Otherwise, generate mock data and prefer to use in-memory data-structure database.
When use sqlite, use local file database (in current directory).

2. You may use libraries like Flask, websocket and etc.
When use Flask, also enable CORS.

3. Do not add authentication or login (or admin account). Make it public accessible and focus on application functionality.

4. Bind to `0.0.0.0:{PORT}`.

Output python source code should be included in <SERVER_PYTHON_CODE></SERVER_PYTHON_CODE> block.
"""

SERVER_UNIT_TESTS_SYSTEM_PROMPT = f"""You are Claude, an AI assistant powered by Anthropic's Claude-3.5-Sonnet model, specialized in backend development.
You are good at writing python unit tests for webservers in single self-contained python files.

Don't add explanation after the generated code.

Output python source code should be included in <SERVER_UNIT_TESTS_PYTHON_CODE></SERVER_UNIT_TESTS_PYTHON_CODE> block.
"""

HTML_SYSTEM_PROMPT = f"""You are an expert in Web development, including CSS, JavaScript, React, Tailwind, Node.JS and Hugo / Markdown. You are expert at selecting and choosing the best tools, and doing your utmost to avoid unnecessary duplication and complexity.
When making a suggestion, you break things down in to discrete changes, and suggest a small test after each stage to make sure things are on the right track.
Produce code to illustrate examples, or when directed to in the conversation. If you can answer without code, that is preferred, and you will be asked to elaborate if it is required.
Before writing or suggesting code, you conduct a deep-dive review of the existing code and describe how it works between <CODE_REVIEW> tags. Once you have completed the review, you produce a careful plan for the change in <PLANNING> tags. Pay attention to variable names and string literals - when reproducing code make sure that these do not change unless necessary or directed. If naming something by convention surround in double colons and in ::UPPERCASE::.
Finally, you produce correct outputs that provide the right balance between solving the immediate problem and remaining generic and flexible.
You always ask for clarifications if anything is unclear or ambiguous. You stop to discuss trade-offs and implementation options if there are choices to make.
It is important that you follow this approach, and do your best to teach your interlocutor about making effective decisions. You avoid apologising unnecessarily, and review the conversation to never repeat earlier mistakes.
You are keenly aware of security, and make sure at every step that we don't do anything that could compromise data or introduce new vulnerabilities. Whenever there is a potential security risk (e.g. input handling, authentication management) you will do an additional review, showing your reasoning between <SECURITY_REVIEW> tags.
Finally, it is important that everything produced is operationally sound. We consider how to host, manage, monitor and maintain our solutions. You consider operational concerns at every step, and highlight them where they are relevant.
Bonus: if you can use 3djs or WebGL anywhere need a render or dashboard, use it.  

Assume the server is already running at `0.0.0.0:{PORT}`, generate html code that connects to the server.  

Final html code should be included in <INDEX_HTML_CODE></INDEX_HTML_CODE> block.
"""

BASH_SYSTEM_PROMPT = """You are Claude, an AI assistant powered by Anthropic's Claude-3.5-Sonnet model, specialized in software development.
You are experts in python 3.11, and familier with popular libraries, and also good at writing linux bash scripts.

You are currently on a task to write "run.sh" that:
1. create a temp venv
2. use pip to install all the required libaries
3. use python to start the webserver. ("python server.py")

(Assume anaconda and pip are installed.)

Generated scripts should be included in <RUN_BASH_CODE></RUN_BASH_CODE> block.

E.g.
<RUN_BASH_CODE>
#!/bin/sh

# Create and activate a virtual environment (optional but recommended)
python3 -m venv venv
source venv/bin/activate

pip install Flask websocket

python server.py
</RUN_BASH_CODE>
"""

DOCKERFILE_SYSTEM_PROMPT = """You are Claude, an AI assistant powered by Anthropic's Claude-3.5-Sonnet model, specialized in software development.
You are experts in python 3.11, and familier with popular libraries, and also good at writing Dockerfile.

You are currently on a task to write "Dockerfile" that:
1. create a temp venv
2. use pip to install all the required libaries
3. use python to start the webserver. ("python server.py")

(Assume anaconda and pip are installed.)

Generated scripts should be included in <RUN_BASH_CODE></RUN_BASH_CODE> block.

E.g.
<RUN_BASH_CODE>
#!/bin/sh

# Create and activate a virtual environment (optional but recommended)
python3 -m venv venv
source venv/bin/activate

pip install Flask websocket

python server.py
</RUN_BASH_CODE>
"""


def generate_design(problem):
    print("Generate system design", flush=True)
    query = f"""# The problem to solve:
<PROBLEM>
{problem}
</PROBLEM>

Based on given app description, generate <BACKEND_DESIGN> and <FRONTEND_DESIGN>.
<BACKEND_DESIGN> will be written in "backend_design.txt".
<FRONTEND_DESIGN> will be written in "frontend_design.txt".
"""
    responses, histories = query_llm(
        query, system_message=DESIGN_SYSTEM_PROMPT, filename="design"
    )
    save_history(histories[0])
    produce_files(responses[0])


def generate_backend():
    backend_design_prompts = assert_file_exists_and_read("backend_design.txt")
    print("Generate server.py", flush=True)
    query = f"""Generate "server.py". Backend design:
```
{backend_design_prompts}
```
"""
    responses, histories = query_llm(
        query,
        system_message=SERVER_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="server.py",
    )
    save_history(histories[0])
    produce_files(responses[0])


def generate_backend_unit_tests():
    backend_design_prompts = assert_file_exists_and_read("backend_design.txt")
    backend_prompts = assert_file_exists_and_read("server.py")
    print("Generate server_test.py", flush=True)
    query = f"""Generate "server_test.py". Backend design:
```
{backend_design_prompts}
```
Backend server implementation:
```
{backend_prompts}
```
"""
    responses, histories = query_llm(
        query,
        system_message=SERVER_UNIT_TESTS_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="server_test.py",
    )
    save_history(histories[0])
    produce_files(responses[0])


def generate_frontend():
    frontend_design_prompts = assert_file_exists_and_read("frontend_design.txt")
    print("Generate index.html", flush=True)
    query = f"""Generate "index.html" that connect to the server. The "index.html" design:
```
{frontend_design_prompts}
```
"""
    responses, histories = query_llm(
        query,
        system_message=HTML_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="index.html",
    )
    save_history(histories[0])
    produce_files(responses[0])


def stylize_frontend():
    print("Stylize index.html", flush=True)
    query = f"""Stylize and make "index.html" beautiful, look production-ready, by using pure CSS website framework "Tailwind CSS"."""
    responses, histories = query_llm(
        query,
        system_message=HTML_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="index.html",
    )
    save_history(histories[0])
    produce_files(responses[0])


def revise_backend():
    print("Modify server.py", flush=True)
    query = 'Modify "server.py" to also serve "index.html" like a static web server.'
    responses, histories = query_llm(
        query,
        system_message=SERVER_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="server.py",
    )
    save_history(histories[0])
    produce_files(responses[0])


def generate_run_bash():
    print("Generate run.sh", flush=True)
    query = f'Generate "run.sh" to pip install required libraries and start the server.'
    responses, histories = query_llm(
        query,
        system_message=BASH_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="run.sh",
    )
    save_history(histories[0])
    produce_files(responses[0])


def generate_dockerfile():
    print("Generate Dockerfile", flush=True)
    current_dir = os.getcwd()
    current_folder_name = os.path.basename(current_dir)
    DOCKERFILE = f"""
<DOCKERFILE>
FROM python:3.10

# Cache dependencies installation
COPY ./pyproject.toml /engy/pyproject.toml

RUN pip install -e /engy

COPY ./src /engy/src

# Install required libraries
RUN pip install Flask Flask-CORS

COPY ./{current_folder_name}/ /app/

WORKDIR /app

CMD [ "python", "server.py" ]
</DOCKERFILE>
"""
    produce_files(DOCKERFILE)


def generate_all(problem):
    generate_design(problem)
    generate_backend()
    generate_frontend()
    stylize_frontend()
    revise_backend()
    generate_run_bash()
    generate_dockerfile()


def regenerate_backend(prompts):
    print("Re-generate server.py", flush=True)
    query = f'{prompts}\n\nRegenerate "server.py".'
    responses, histories = query_llm(
        query,
        system_message=SERVER_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="server.py",
    )
    save_history(histories[0])
    produce_files(responses[0])


def regenerate_frontend(prompts):
    print("Re-generate index.html", flush=True)
    query = f'{prompts}\n\nRegenerate "index.html".'
    responses, histories = query_llm(
        query,
        system_message=HTML_SYSTEM_PROMPT,
        previous_history=load_history(),
        filename="index.html",
    )
    save_history(histories[0])
    produce_files(responses[0])


def regenerate_all(prompts):
    regenerate_backend(prompts)
    regenerate_frontend("According the changes to the server.")



import re

_PRODUCE_FILE_MAPPING = {
    "AGENTS_DESIGN": "agents_design.json",
    "TASKS_DESIGN": "tasks_design.json",
    "SERVER_PYTHON_CODE": "server.py",
    "INDEX_HTML_CODE": "index.html",
    "RUN_BASH_CODE": "run.sh",
    "BACKEND_DESIGN": "backend_design.txt",
    "FRONTEND_DESIGN": "frontend_design.txt",
    "AGENTS_PYTHON_CODE": "agents.py",
    "DOCKERFILE": "Dockerfile",
    "RUN_DOCKER_BASH": "run_docker.sh",
    "SERVER_UNIT_TESTS_PYTHON_CODE": "server_test.py",
}


def parse_block(block_name, response_text):
    pattern = f"<{block_name}>(.*?)</{block_name}>"
    match = re.search(pattern, response_text, re.DOTALL)
    return match.group(1) if match else None


def produce_files(response_text):
    # First, process the predefined mappings
    for block_name, filename in _PRODUCE_FILE_MAPPING.items():
        block_content = parse_block(block_name, response_text)
        if block_content is not None:
            with open(filename, "w") as f:
                f.write(block_content)
            print(f"New file written: {filename}")

    # Then, look for any other blocks with the format <anyname_extension>
    pattern = r"<(\w+)_(\w+)>(.*?)</\1_\2>"
    matches = re.finditer(pattern, response_text, re.DOTALL)

    for match in matches:
        block_name = match.group(1)
        extension = match.group(2)
        content = match.group(3)

        # Check if this block name is not in the predefined mapping
        if f"{block_name.upper()}_{extension.upper()}" not in _PRODUCE_FILE_MAPPING:
            filename = f"{block_name}.{extension}"
            with open(filename, "w") as f:
                f.write(content)
            print(f"New file written: {filename}")

    print("Files have been produced based on the response text.")



from .code_map import load_code_folder_to_system_prompt
from .llm import query_llm
from .produce_files import produce_files
from .util import load_history, save_history


def split_frontend():
    print("Splitting frontend into modular files", flush=True)
    
    # Load the current state of the project
    system_prompt = load_code_folder_to_system_prompt('.')
    
    query = '''Split the "index.html" into smaller, modular ".css", ".js", ".html" files.
Each file should preferably be less than 100 lines.

Use semantic naming for the new files.
Update the main HTML file to properly link the new CSS and JS files.
Output each new file content in the format <filename_extension>content</filename_extension>.'''
    
    responses, histories = query_llm(
        query,
        system_message=system_prompt,
        previous_history=load_history(prefix="split"),
        filename="frontend_split"
    )
    save_history(histories[0], prefix="split")
    produce_files(responses[0])


def split_backend():
    print("Splitting backend into modular files", flush=True)
    
    # Load the current state of the project
    system_prompt = load_code_folder_to_system_prompt('.')
    
    query = '''Split the "server.py" into smaller, modular ".py" files.
Each file should preferably be less than 100 lines.
Also update "server.py" to be able to serve ".js" and ".css" files.

Use semantic naming for the new files.
Update the main server file to properly import and use the new modules.
Output each new file content in the format <filename_extension>content</filename_extension>.'''
    
    responses, histories = query_llm(
        query,
        system_message=system_prompt,
        previous_history=load_history(prefix="split"),
        filename="backend_split"
    )
    save_history(histories[0], prefix="split")
    produce_files(responses[0])


def split():
    split_frontend()
    split_backend()



import os
import shutil
import unittest
from pathlib import Path
from textwrap import dedent

from .produce_files import produce_files


class TestProduceFiles(unittest.TestCase):
    def setUp(self):
        # Create a temporary directory for test files
        self.test_dir = Path("test_output")
        self.test_dir.mkdir(exist_ok=True)
        os.chdir(self.test_dir)

    def tearDown(self):
        # Change back to the parent directory and remove the test directory
        os.chdir("..")
        shutil.rmtree(self.test_dir)

    def test_predefined_mapping(self):
        test_content = dedent("""
        <AGENTS_DESIGN>
        {"agent": "test"}
        </AGENTS_DESIGN>
        """)
        produce_files(test_content)
        self.assertTrue(Path("agents_design.json").exists())
        with open("agents_design.json", "r") as f:
            self.assertEqual(f.read().strip(), '{"agent": "test"}')

    def test_custom_block(self):
        test_content = dedent("""
        <custom_script_py>
        print("Hello, World!")
        </custom_script_py>
        """)
        produce_files(test_content)
        self.assertTrue(Path("custom_script.py").exists())
        with open("custom_script.py", "r") as f:
            self.assertEqual(f.read().strip(), 'print("Hello, World!")')

    def test_multiple_blocks(self):
        test_content = dedent("""
        <AGENTS_DESIGN>
        {"agent": "test"}
        </AGENTS_DESIGN>
        <custom_script_py>
        print("Hello, World!")
        </custom_script_py>
        <another_file_txt>
        This is another test file.
        </another_file_txt>
        """)
        produce_files(test_content)
        self.assertTrue(Path("agents_design.json").exists())
        self.assertTrue(Path("custom_script.py").exists())
        self.assertTrue(Path("another_file.txt").exists())

    def test_no_matching_blocks(self):
        test_content = "This content has no blocks to produce files."
        produce_files(test_content)
        self.assertEqual(len(list(Path().glob("*"))), 0)

    def test_empty_block(self):
        test_content = dedent("""
        <empty_file_txt></empty_file_txt>
        """)
        produce_files(test_content)
        self.assertTrue(Path("empty_file.txt").exists())
        with open("empty_file.txt", "r") as f:
            self.assertEqual(f.read(), "")


if __name__ == "__main__":
    unittest.main()



import unittest
import tempfile
import os
from pathlib import Path
from textwrap import dedent

# Assuming the original function is in a file named 'code_loader.py'
from .code_map import load_code_folder_to_system_prompt

class TestLoadCodeFolderToSystemPrompt(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.TemporaryDirectory()
        self.addCleanup(self.temp_dir.cleanup)

        # Create example files
        self.create_file('main.py', 'print("Hello, World!")')
        self.create_file('styles.css', 'body { color: blue; }')
        self.create_file('script.js', 'console.log("JavaScript");')
        self.create_file('index.html', '<h1>Hello</h1>')
        self.create_file('setup.sh', 'echo "Setup complete"')
        self.create_file('readme.txt', 'This file should be ignored')

    def create_file(self, filename, content):
        path = Path(self.temp_dir.name) / filename
        path.write_text(content)

    def test_load_code_folder_to_system_prompt(self):
        result = load_code_folder_to_system_prompt(self.temp_dir.name)

        # Check if all expected files are included
        self.assertIn('main.py', result)
        self.assertIn('styles.css', result)
        self.assertIn('script.js', result)
        self.assertIn('index.html', result)
        self.assertIn('setup.sh', result)

        # Check if ignored file is not included
        self.assertNotIn('readme.txt', result)

        # Check if file contents are correctly included
        self.assertIn('print("Hello, World!")', result)
        self.assertIn('body { color: blue; }', result)
        self.assertIn('console.log("JavaScript")', result)
        self.assertIn('<h1>Hello</h1>', result)
        self.assertIn('echo "Setup complete"', result)

        # Check if the output format instructions are included
        self.assertIn('<filename_extension>File content goes here</filename_extension>', result)
        self.assertIn('<myfile_py>print(\'Hello, World!\')</myfile_py>', result)

        # Check if the objective and notes are included
        self.assertIn('# Objective', result)
        self.assertIn('# Output', result)
        self.assertIn('## Note', result)

    def test_empty_folder(self):
        empty_dir = tempfile.TemporaryDirectory()
        self.addCleanup(empty_dir.cleanup)

        result = load_code_folder_to_system_prompt(empty_dir.name)

        # Check if the basic structure is still there
        self.assertIn('# Example Project', result)
        self.assertIn('# Objective', result)
        self.assertIn('# Output', result)

        # Check that no file contents are included
        self.assertNotIn('```', result)

if __name__ == '__main__':
    unittest.main()


import os
import random
import shutil

from .code_map import load_code_folder_to_system_prompt
from .llm import auto_load_dotenv, query_llm
from .produce_files import produce_files
from .util import load_history, save_history

PORT = random.randint(5000, 10000)


def generate_backend(prompts, system_prompts):
    print('Generate server.py', flush=True)
    query = f'''<FEATURE_REQUEST>
{prompts}
</FEATURE_REQUEST>

Generate new "server.py" first. Re-assign a new PORT, to "0.0.0.0:{PORT}".
'''
    responses, histories = query_llm(
        query, system_message=system_prompts, previous_history=load_history(), filename='server.py')
    save_history(histories[0])
    produce_files(responses[0])


def generate_frontend(system_prompts):
    print('Generate index.html', flush=True)
    query = 'Generate new "index.html"'
    responses, histories = query_llm(
        query, system_message=system_prompts, previous_history=load_history(), filename='index.html')
    save_history(histories[0])
    produce_files(responses[0])


def generate_run_bash(system_prompts):
    print('Generate run.sh', flush=True)
    query = 'Generate new "run.sh". Only add new missing install, no need to make big changes.'
    responses, histories = query_llm(
        query, system_message=system_prompts, previous_history=load_history(), filename='run.sh')
    save_history(histories[0])
    produce_files(responses[0])


def clone_all(path, prompts):
    try:
        shutil.copy2(os.path.join(path, '.env'), '.env')
        auto_load_dotenv()
    except (FileNotFoundError, shutil.SameFileError, OSError):
        pass  # Silently ignore errors

    system_prompts = load_code_folder_to_system_prompt(path)
    generate_backend(prompts, system_prompts)
    generate_frontend(system_prompts)
    generate_run_bash(system_prompts)



import argparse
import os
import subprocess
import sys
from contextlib import contextmanager

import yaml

from .app_builder import (
    generate_all,
    generate_backend_unit_tests,
    generate_dockerfile,
    regenerate_all,
    regenerate_backend,
    regenerate_frontend,
)
from .app_cloner import clone_all
from .app_split import split
from .util import assert_file_exists_and_read, run_docker


@contextmanager
def log_to_file(should_log):
    if should_log:
        log_file = open("terminal.log", "a")
        original_stdout = sys.stdout
        sys.stdout = log_file
    try:
        yield
    finally:
        if should_log:
            sys.stdout = original_stdout
            log_file.close()


def diff_list(current_list, goal_list):
    current_list = current_list or []
    goal_list = goal_list or []
    current_set = set(current_list)
    goal_set = set(goal_list)
    added = goal_set - current_set
    removed = current_set - goal_set
    return list(added), list(removed)


def diff_configs():
    if os.path.exists("config.yaml"):
        with open("config.yaml", "r") as file:
            goal_config = yaml.safe_load(file)
    else:
        goal_config = {}

    if os.path.exists("state.yaml"):
        with open("state.yaml", "r") as file:
            current_config = yaml.safe_load(file)
    else:
        current_config = {}

    new_features, removed_features = diff_list(
        current_config.get("features", []), goal_config.get("features", [])
    )
    new_bugs, removed_bugs = diff_list(
        current_config.get("bugs", []), goal_config.get("bugs", [])
    )
    return new_features, removed_features, new_bugs, removed_bugs


def main():
    parser = argparse.ArgumentParser(
        description="Generate python webapp from prompts. (require either `input.txt` or `config.yaml`)"
    )
    parser.add_argument(
        "--log-to-file", action="store_true", help="Log output to terminal.log"
    )
    subparsers = parser.add_subparsers(dest="subcommand", help="Subcommands")

    backend_parser = subparsers.add_parser("backend", help="Regenerate the backend")
    frontend_parser = subparsers.add_parser("frontend", help="Regenerate the frontend")
    bug_parser = subparsers.add_parser("bug", help="Fix the bug in bug.txt")
    feature_parser = subparsers.add_parser(
        "feature", help="Add the feature stated in feature.txt"
    )
    clone_parser = subparsers.add_parser(
        "clone", help="Clone a new engy app based on existing engy app"
    )
    clone_parser.add_argument("path", type=str, help="path to clone")
    diff_parser = subparsers.add_parser(
        "diff", help="Diff between current and goal configurations (config.yaml)"
    )
    apply_parser = subparsers.add_parser(
        "apply", help="Apply diff to match goal configurations (config.yaml)"
    )
    docker_parser = subparsers.add_parser("docker", help="Generate Dockerfile")
    run_docker_parser = subparsers.add_parser("run_docker", help="Generate Dockerfile")
    refactor_parser = subparsers.add_parser(
        "refactor", help="Refactor the frontend and backend into modular files"
    )
    split_parser = subparsers.add_parser(
        "split", help="Split the frontend and backend into modular files"
    )
    backend_unit_tests_parser = subparsers.add_parser(
        "unit_tests", help="Generate backend unit tests"
    )

    args = parser.parse_args()

    with log_to_file(args.log_to_file):
        if args.subcommand == "backend":
            prompts = input("Input prompts:")
            regenerate_backend(prompts)
        elif args.subcommand == "frontend":
            prompts = input("Input prompts:")
            regenerate_frontend(prompts)
        elif args.subcommand == "bug":
            bug_description = assert_file_exists_and_read("bug.txt")
            prompts = f"Fix bug:\n<BUG>\n{bug_description}\n</BUG>"
            regenerate_all(prompts)
        elif args.subcommand == "feature":
            feature_description = assert_file_exists_and_read("feature.txt")
            prompts = f"<FEATURE_REQUEST>\n{feature_description}\n</FEATURE_REQUEST>"
            regenerate_all(prompts)
        elif args.subcommand == "clone":
            print(f"Clone project {args.path}")
            if not os.path.exists("input.txt"):
                parser.print_help()
                sys.exit(1)
            input_prompts = assert_file_exists_and_read("input.txt")
            clone_all(args.path, input_prompts)
        elif args.subcommand == "diff":
            print("Diff configs")
            new_features, removed_features, new_bugs, removed_bugs = diff_configs()
            print("Adding features:", new_features)
            print("Removing features:", removed_features)
            print("Adding bugs:", new_bugs)
        elif args.subcommand == "apply":
            print("Apply diff")
            new_features, removed_features, new_bugs, removed_bugs = diff_configs()
            print("Adding features:", new_features)
            print("Removing features:", removed_features)
            print("Adding bugs:", new_bugs)
            bugs_prompt = "\n".join(
                [f"Fix bug:\n<BUG>\n{bug}\n</BUG>" for bug in new_bugs]
            )
            add_features_prompt = "\n".join(
                [
                    f"<FEATURE_REQUEST>\n{feature}\n</FEATURE_REQUEST>"
                    for feature in new_features
                ]
            )
            remove_features_prompt = "\n".join(
                [
                    f"<FEATURE_REQUEST>\nRemove feature '{feature}'\n</FEATURE_REQUEST>"
                    for feature in removed_features
                ]
            )
            prompts = bugs_prompt + add_features_prompt + remove_features_prompt
            print(f"{prompts=}")
            regenerate_all(prompts)
            with open("state.yaml", "w") as file:
                yaml.dump(
                    {"features": new_features, "bugs": new_bugs},
                    file,
                    default_flow_style=False,
                )
        elif args.subcommand == "docker":
            generate_dockerfile()
        elif args.subcommand == "run_docker":
            run_docker()
        elif args.subcommand == "unit_tests":
            generate_backend_unit_tests()
        elif args.subcommand == "split":
            split()
        else:
            print("App builder")
            if not os.path.exists("input.txt"):
                parser.print_help()
                sys.exit(1)
            input_prompts = assert_file_exists_and_read("input.txt")
            generate_all(input_prompts)


if __name__ == "__main__":
    main()



import os
import pickle
import subprocess
from pathlib import Path
from typing import List

from .llm import ChatMessage


def assert_file_exists_and_read(filename):
    # Assert that the file exists
    assert os.path.exists(filename), f"File '{filename}' does not exist"

    # Read the file content
    try:
        with open(filename, "r") as file:
            content = file.read()
        return content
    except Exception as e:
        assert False, f"Failed to read file '{filename}': {str(e)}"


def save_history(history, prefix="default"):
    history_filename = Path(f"{prefix}.history.pickle")
    with open(history_filename, "wb") as f:
        return pickle.dump(history, f)


def load_history(prefix="default") -> List[ChatMessage]:
    history_filename = Path(f"{prefix}.history.pickle")
    if history_filename.exists():
        return pickle.loads(history_filename.read_bytes())
    return []


def run_process(cmd):
    process = subprocess.Popen(
        cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        bufsize=1,
    )

    for line in iter(process.stdout.readline, ""):
        print(line, end="")

    exit_code = process.wait()
    return exit_code


def run_docker():
    current_dir = os.getcwd()
    current_folder_name = os.path.basename(current_dir)
    cmd = f"docker build -f Dockerfile -t {current_folder_name}:latest ../ && docker run --network host {current_folder_name}:latest"
    return run_process(cmd)



from typing import Callable
from crewai_tools import ScrapeWebsiteTool

scrape_tool = ScrapeWebsiteTool()


def web_scraper_agent(url: str, producer: Callable[[dict], None]):
    content = scrape_tool.run(website_url=url)
    producer({'url': url, 'webpage_content': content})


def _terminal_producer(entity):
    print(f"Produced entity: {entity}")


if __name__ == '__main__':
    web_scraper_agent('https://engy.ai', _terminal_producer)
